///usr/bin/env jbang "$0" "$@" ; exit $?

//DEPS com.google.protobuf:protobuf-java:3.24.3
//SOURCES SevenSegmentOuterClass.java

import java.io.ByteArrayOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.HexFormat;

import com.google.protobuf.*;
import code.SevenSegmentOuterClass.SevenSegments.SevenSegment;

/**
 * This example must be executed with JBang to download and include the dependencies.
 * jbang ProtoBufferExample.java
 */
public class ProtoBufferExample {
    public static void main(String[] args) throws IOException {
        SevenSegment.Builder sevenSegment = SevenSegment.newBuilder();
        sevenSegment.setCenterTop(true);
        sevenSegment.setLeftTop(true);
        sevenSegment.setCenterMiddle(true);
        sevenSegment.setRightBottom(true);
        sevenSegment.setCenterBottom(true);

        // Compress
        long startTime = System.nanoTime();
        ByteArrayOutputStream output = new ByteArrayOutputStream();
        sevenSegment.build().writeTo(output);
        var compressed = output.toString();
        output.close();
        long endTime = System.nanoTime();

        System.out.println("----------------------------------------------------------------------");

        System.out.println("Data as HEX: #" + HexFormat.of().formatHex(output.toByteArray()));
        System.out.println("Compressed: " + compressed.length()
            + "\tDuration: " + ((endTime - startTime) / 1_000_000F) + "ms");

        System.out.println("----------------------------------------------------------------------");

        // Decompress
        startTime = System.nanoTime();
        SevenSegment sevenSegmentFromData = SevenSegment.parseFrom(output.toByteArray());
        endTime = System.nanoTime();

        System.out.println("Decompressed duration: " + ((endTime - startTime) / 1_000_000F) + "ms");

        System.out.println("----------------------------------------------------------------------");

        System.out.println("              \tOrigin\tDecompr");
        System.out.println("Center top:   \t"
                + sevenSegment.getCenterTop()
                + "\t" + sevenSegmentFromData.getCenterTop());
        System.out.println("Center middle:\t"
                + sevenSegment.getCenterMiddle()
                + "\t" + sevenSegmentFromData.getCenterMiddle());
        System.out.println("Center bottom:\t"
                + sevenSegment.getCenterBottom()
                + "\t" + sevenSegmentFromData.getCenterBottom());
        System.out.println("Left top:     \t"
                + sevenSegment.getLeftTop()
                + "\t" + sevenSegmentFromData.getLeftTop());
        System.out.println("Left bottom:  \t"
                + sevenSegment.getLeftBottom()
                + "\t" + sevenSegmentFromData.getLeftBottom());
        System.out.println("Right top:    \t"
                + sevenSegment.getRightTop()
                + "\t" + sevenSegmentFromData.getRightTop());
        System.out.println("Right bottom: \t"
                + sevenSegment.getRightBottom()
                + "\t" + sevenSegmentFromData.getRightBottom());
        System.out.println("Dot:          \t"
                + sevenSegment.getDot()
                + "\t" + sevenSegmentFromData.getDot());

        System.out.println("----------------------------------------------------------------------");
    }

    private static final String json = """
            {
                "leftTop": true,
                "leftBottom": false,
                "rightTop": false,
                "rightBottom": true,
                "centerTop": true,
                "centerMiddle": true,
                "centerBottom": true,
                "dot": false
            }
            """;
}